local mods_for_p1= GAMESTATE:IsPlayerEnabled(PLAYER_1) and GAMESTATE:GetCurrentSteps(PLAYER_1):GetDifficulty()=="Difficulty_Beginner"
local mods_for_p2= GAMESTATE:IsPlayerEnabled(PLAYER_2) and GAMESTATE:GetCurrentSteps(PLAYER_2):GetDifficulty()=="Difficulty_Beginner"
local mods_for_either= mods_for_p1 or mods_for_p2

local function optional_actor(frame, cond, actor)
	if cond then
		frame[#frame+1]= actor
	end
end

local divider= {}
local poptions= {}
local function set_divider_from_mods(pn)
	local tilt= poptions[pn]:Tilt()
	local skew= poptions[pn]:Skew()
	if skew == 0 then
		if tilt < 0 then
			-- hallway
			divider[pn]= .94
		elseif tilt > 0 then
			-- distant
			divider[pn]= 1.15
		else
			-- overhead
			divider[pn]= 1
		end
	else
		if tilt < 0 then
			-- incoming
			divider[pn]= .97
		elseif tilt > 0 then
			-- space
			divider[pn]= 1
		else
			-- silly
			divider[pn]= 1
		end
	end
end

for i, pn in ipairs(GAMESTATE:GetEnabledPlayers()) do
	poptions[pn]= GAMESTATE:GetPlayerState(pn):GetPlayerOptions("ModsLevel_Song")
	set_divider_from_mods(pn)
end

local function apply_mod(str, pn)
	if pn then
		poptions[pn]:FromString(str)
	else
		for pn, ops in pairs(poptions) do
			ops:FromString(str)
		end
	end
end

local setMod= {}
local FoxDir = 0
local Transitioner = 0.24
local TotalDistance = 8

local function asteroid(tex, roty, zrotmin, move)
	return Def.Sprite{
		Name= "asteroid",
		Texture= tex,
		InitCommand= function(self)
			self:diffuse(0.05,0.05,0.05,1)
			self:rotationz(0)
			self:rotationy(roty)
			self:x(SCREEN_CENTER_X)
			self:y(SCREEN_CENTER_Y)
			self:zoom(0.1)
			self:accelerate(math.random(2,3))
			self:playcommand("Move")
			self:zoom(math.random(2,5))
			self:rotationz(math.random(zrotmin,60))
			self:diffuse(0.3,0.3,0.3,1)
			self:queuecommand('Init')
		end,
		MoveCommand= move,
	}
end

local asteroid_frame= Def.ActorFrame{
	InitCommand= cmd(wag;effectmagnitude,4,4,4),
	asteroid(
		"Asteroid1.png", 0, 10, function(self)
			self:addx(math.random(350,450))
			self:addy(math.random(300,400))
	end),
	asteroid(
		"Asteroid2.png", 180, -60, function(self)
			self:addx(math.random(-100,0))
			self:addx(-350)
			self:addy(math.random(-100,0))
			self:addy(-300)
	end),
	asteroid(
		"Asteroid1.png", 180, 10, function(self)
			self:addx(math.random(350,450))
			self:addy(math.random(-100,0))
			self:addy(-300)
	end),
	asteroid(
		"Asteroid2.png", 0, -60, function(self)
			self:addx(math.random(-100,0))
			self:addx(-350)
			self:addy(math.random(300,400))
	end),
	asteroid(
		"Asteroid1.png", 0, -60, function(self)
			self:addx(math.random(350,450))
			self:addy(math.random(300,400))
	end),
	asteroid(
		"Asteroid2.png", 0, -60, function(self)
			self:addx(math.random(-100,0))
			self:addx(-350)
			self:addy(math.random(-100,0))
			self:addy(-300)
	end),
	asteroid(
		"Asteroid1.png", 180, -60, function(self)
			self:addx(math.random(350,450))
			self:addy(math.random(-100,0))
			self:addy(-300)
	end),
	asteroid(
		"Asteroid2.png", 0, 10, function(self)
			self:addx(math.random(350,450))
			self:addy(math.random(300,400))
	end),
}



local frame= Def.ActorFrame{
  Name= "actor_d",
  InitCommand= function(self)
	end,
}

optional_actor(frame, mods_for_either, asteroid_frame)

local ship_stuff_frame= Def.ActorFrame{
	Name= "actor_o",
	InitCommand= function(self)
		self:x(SCREEN_CENTER_X-32)
		self:y(SCREEN_CENTER_Y+100)
		self:zoom(2)
	end,
	NoteCrossedMessageCommand= function(self, params)
		if GAMESTATE:GetSongBeat() > 0 and GAMESTATE:GetSongBeat() < 5 then
			SCREENMAN:GetTopScreen():GetChild('Underlay'):accelerate(0.5)
			SCREENMAN:GetTopScreen():GetChild('Underlay'):addy(-2000)

		end
		if params.ButtonName then
			self:playcommand("NoteCrossed"..params.ButtonName)
		end
	end,
	NoteCrossedDownMessageCommand= function(self)
		FoxDir = 2
		self:stoptweening()
		self:linear(Transitioner*2)
		self:addy(TotalDistance)
		self:addy(TotalDistance)
		self:sleep(0.0001)
	end,
	NoteCrossedLeftMessageCommand= function(self)
		FoxDir = 1
		self:stoptweening();			
		self:linear(Transitioner*2)
		self:addx(-TotalDistance*1.325)
		self:addx(-TotalDistance*1.325)
		self:sleep(0.0001)
	end,
	NoteCrossedRightMessageCommand= function(self)
		FoxDir = 4
		self:stoptweening()
		self:linear(Transitioner*2)
		self:addx(TotalDistance)
		self:addx(TotalDistance)
		self:sleep(0.0001)
	end,
	NoteCrossedUpMessageCommand= function(self)
		FoxDir = 3
		self:stoptweening()
		self:linear(Transitioner*2)
		self:addy(-TotalDistance)
		self:addy(-TotalDistance)
		self:sleep(0.0001)
	end,
	Def.Sprite{
		Name= "black",
		InitCommand= cmd(zoom,640;diffusealpha,0;linear,0.5;diffusealpha,1;sleep,1;linear,0.7;diffusealpha,0;sleep,2000),
		Texture= "black.png",
	},
}

local function player_initter(pn)
	-- Yes, Jayce's logic only cleared mods if you didn't set an xmod,
	-- and set C620 when it did.  No, I don't know why it didn't default
	-- to an xmod.  -Kyz
	setMod[pn]= poptions[pn]:XMod()
	if type(setMod[pn]) ~= "number" then
		setMod[pn]= 620
		poptions[pn]:FromString('clearall')
		poptions[pn]:CMod(620, 100)
	else
		poptions[pn]:XMod((setMod[pn] + .25) * divider[pn], 100)
	end
	apply_mod('*20000 -5% bumpy, *20000 150% distant', pn)
	return Def.Actor{
		NoteCrossedMessageCommand= function(self)
			if GAMESTATE:GetSongBeat() < 5 then
				local player= SCREENMAN:GetTopScreen():GetChild('Player'..ToEnumShortString(pn))
				player:accelerate(0.5):addy(5)
					:decelerate(.5):addy(-5)
			end
		end,
	}
end

for i, pn in ipairs(GAMESTATE:GetEnabledPlayers()) do
	ship_stuff_frame[#ship_stuff_frame+1]= player_initter(pn)
end

local ship_frame= Def.ActorFrame{
	Name= "actor_r",
	NoteCrossedJumpMessageCommand= function(self)
		Decider = math.random(0,1)
		FoxDir = 0
		self:stoptweening()
		if Decider == 0 then
			self:rotationz(0)
			self:decelerate(Transitioner)
			self:rotationz(360)
		end
		if Decider == 1 then
			self:rotationz(0)
			self:decelerate(Transitioner)
			self:rotationz(-360)
		end
	end,
}

local airwing_anims= {
	-- idle
	[0]= {
		{Frame= 0, Delay= 0.08},
		{Frame= 1, Delay= 0.08},
	},
	-- left
	{
		{Frame= 2, Delay= 0.04},
		{Frame= 3, Delay= 0.08},
		{Frame= 4, Delay= 0.04},
		{Frame= 3, Delay= 0.08},
		{Frame= 2, Delay= 0.08},
		{Frame= 0, Delay= 20},
	},
	-- down
	{
		{Frame= 10, Delay= 0.04},
		{Frame= 11, Delay= 0.08},
		{Frame= 12, Delay= 0.04},
		{Frame= 11, Delay= 0.08},
		{Frame= 10, Delay= 0.08},
		{Frame= 0, Delay= 20},
	},
	-- up
	{
		{Frame= 14, Delay= 0.04},
		{Frame= 15, Delay= 0.08},
		{Frame= 16, Delay= 0.04},
		{Frame= 15, Delay= 0.08},
		{Frame= 14, Delay= 0.08},
		{Frame= 0, Delay= 20},
	},
	-- right
	{
		{Frame= 6, Delay= 0.04},
		{Frame= 7, Delay= 0.08},
		{Frame= 8, Delay= 0.04},
		{Frame= 7, Delay= 0.08},
		{Frame= 6, Delay= 0.08},
		{Frame= 0, Delay= 20},
	},
}

local airwing= Def.Sprite{
	Texture= "airwing 6x3.png",
	Frames= airwing_anims[0],
	InitCommand= cmd(wag;effectmagnitude,5,5,5;diffusealpha,0;sleep,1;diffusealpha,1),
	do_stepCommand= function(self)
		if GAMESTATE:GetSongBeat() > 0 then
			self:SetStateProperties(airwing_anims[FoxDir])
		end
	end,
	StepP1MessageCommand= function(self)
		self:playcommand("do_step")
	end,
	StepP2MessageCommand= function(self)
		self:playcommand("do_step")
	end,
}

optional_actor(ship_frame, mods_for_either, airwing)

ship_stuff_frame[#ship_stuff_frame+1]= ship_frame

frame[#frame+1]= Def.ActorFrame{
	Name= "actor_n",
	InitCommand= function(self)
		self:sleep(179.027)
		self:accelerate(1.5)
		self:zoom(0.01)
		self:diffusealpha(0)
		self:x(SCREEN_CENTER_X)
		self:y(SCREEN_CENTER_Y)
		self:sleep(0.001)
		self:addx(20000)
		self:sleep(20)
	end,
	ship_stuff_frame,
}

return frame
